/* Copyright (c) 2008 Markus Peloquin <markus@cs.wisc.edu>
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE. */

#include "field.hpp"
#include "person.hpp"

void
Field::find_close()
{
	for (size_t c = 0; c < _cols; c++) {
		for (size_t r = 0; r < _rows; r++) {
			Node &node = _field[index_of(r, c)];
			std::auto_ptr<Search_area> area = search_radius(r, c);
			for (Search_area::iterator i = area->indices.begin();
			    i != area->indices.end(); i++) {
				consider(node, _field[*i]);
				
			}
		}
	}
}

void
Field::consider(Node &n1, const Node &n2)
{
	for (Node::iterator i = n1.people.begin(); i != n1.people.end(); ++i)
		for (Node::const_iterator j = n2.people.begin();
		    j != n2.people.end(); ++j)
			if (i != j)
				(*i)->add_if_close(*j);
}

inline size_t
diff(size_t a, size_t b)
{
	return a < b ? b - a : a - b;
}

std::auto_ptr<Field::Search_area>
Field::search_radius(size_t row, size_t col) const
{
	std::auto_ptr<Search_area> area(new Search_area());

	size_t index = index_of(row, col);
	size_t count = _field[index].people.size() - 1;
	area->indices.push_back(index);

	size_t r, c;
	size_t rmin = row;
	size_t cmin = col;
	size_t rmax = row + 1;
	size_t cmax = col + 1;

	size_t rad = 0;
	size_t dist = 0;
	size_t distprev;
	size_t cdist, rdist;

	short runs_left = 2;
	while (true) {
		if (count >= CLOSE_FRIENDS) {
			if (!runs_left--)
				break;
		}
		distprev = dist;

		rad++;
		dist = rad * rad;
		dist *= dist;

		if (rmin) rmin--;
		if (cmin) cmin--;
		if (rmax != _rows) rmax++;
		if (cmax != _cols) cmax++;

		for (c = cmin; c < cmax; c++) {
			cdist = diff(c, col);
			cdist *= cdist;
			for (r = rmin; r < rmax; r++) {
				rdist = diff(r, row);
				rdist *= rdist;
				if (rdist + cdist > distprev &&
				    rdist + cdist <= dist) {
					// in radius, not counted before
					index = index_of(r, c);
					count += _field[index].people.size();
					area->indices.push_back(index);
				}
			}
		}
	}

	return area;
}
