/* Copyright (c) 2008 Markus Peloquin <markus@cs.wisc.edu>
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE. */

#include <algorithm>

#include "person.hpp"

using std::vector;

// predicate for find_if()
struct Closer_than : std::unary_function<const Neighbor &, bool> {
	Closer_than(double dist) : dist(dist) {}
	bool operator()(const Neighbor &n)
	{
		return n.d < dist;
	}
	double dist;
};

void
Person::add_if_close(std::tr1::shared_ptr<const Person> p, double dist)
{
	Neighbor n = { p, dist };

	if (_neighbors.empty()) {
		_neighbors.push_back(n);
		return;
	}

	vector<Neighbor>::iterator pos =
	    find_if(_neighbors.begin(), _neighbors.end(), Closer_than(dist));

	if (_neighbors.size() < CLOSE_FRIENDS) {
		_neighbors.insert(pos, n);
	} else if (pos != _neighbors.begin()) {
		// already three friends, and pos points to a neighbor
		// that is closer (or is end())
		_neighbors.insert(pos, n);

		_neighbors.erase(_neighbors.begin());
	}
}
