/* Copyright (c) 2008 Markus Peloquin <markus@cs.wisc.edu>
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE. */

#ifdef TIMER
#   include <iomanip>
#endif
#include <iostream>
#include <fstream>
#include <functional>
#include <list>
#include <sstream>
#include <tr1/memory>
#include <vector>

#ifdef TIMER
#   include <boost/timer.hpp>
#endif

#include "field.hpp"
#include "parse_error.hpp"
#include "person.hpp"

using std::list;
using std::string;
using std::tr1::shared_ptr;
using std::vector;

void parse_input(const string &path, std::ifstream &file,
    vector<shared_ptr<Person> > *friends)
{
	string line;
	size_t linenum = 0;
	char nothing;
	while (std::getline(file, line)) {
		std::istringstream in(line);
		linenum++;

		int id;
		double lat;
		double lon;
		in >> id >> lat >> lon;
		if (!in) throw Parse_error(path, linenum);

		// make sure there is nothing else
		in >> nothing;
		if (in) throw Parse_error(path, linenum);

		friends->push_back(shared_ptr<Person>(
		    new Person(id, lat, lon)));
	}
}

void print_close(std::ostream &out, const vector<shared_ptr<Person> > &friends)
{
	typedef vector<shared_ptr<Person> >::const_iterator person_iter;
	typedef vector<Neighbor>::const_reverse_iterator neighbor_iter;

	for (person_iter i = friends.begin(); i != friends.end(); ++i) {
		const Person &p = **i;
		out << p.id() << ' ';

		neighbor_iter first = p.neighbors().rbegin();
		neighbor_iter last = p.neighbors().rend();
		for (neighbor_iter j = first; j != last; ++j) {
			if (j != first)
				out << ',';
			out << j->p->id();
		}
		out << '\n';
	}
}

int main(int argc, char **argv)
{
#ifdef TIMER
	boost::timer timer;
	std::ofstream devnull("/dev/null");
#endif

	if (argc != 2) {
		std::cout << "usage: " << *argv << " FILE\n";
		return 1;
	}

	string path(argv[1]);
	std::ifstream infile(argv[1]);
	vector<shared_ptr<Person> > friends;

	parse_input(path, infile, &friends);

	Field field(friends.begin(), friends.end());
	field.find_close();

#ifdef TIMER
	print_close(devnull, friends);

	std::cout << "elapsed time: " << std::scientific
	    << timer.elapsed() << " sec\n";
#else
	print_close(std::cout, friends);
#endif

	return 0;
}
